//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_SERVICE_H
#define FGGL_SERVICE_H

#include <memory>
#include <typeindex>
#include <unordered_map>

namespace fggl::util {

	class ServiceLocator {
			inline static ServiceLocator *s_instance;
			std::unordered_map<std::type_index, std::shared_ptr<void>> m_services;
			ServiceLocator() = default;

		public:
			// don't allow copying or moving
			ServiceLocator(ServiceLocator &other) = delete;
			ServiceLocator(ServiceLocator &&other) = delete;

			static ServiceLocator &instance() {
				if (s_instance == nullptr) {
					s_instance = new ServiceLocator();
				}
				return *s_instance;
			}

			template<typename T>
			std::shared_ptr<T> supply(std::shared_ptr<T> ptr) {
				auto info = std::type_index(typeid(T));
				m_services[info] = ptr;
				return ptr;
			}

			template<typename T>
			std::shared_ptr<T> get() {
				auto info = std::type_index(typeid(T));
				return std::static_pointer_cast<T>(m_services.at(info));
			}

			template<typename T>
			std::shared_ptr<T> providePtr() {
				auto info = std::type_index(typeid(T));
				return std::static_pointer_cast<T>(m_services.at(info));
			}

	};

}

#endif //FGGL_SERVICE_H
